function renderToast(msg, color) {
  var toast = document.getElementById("snackbar");

  toast.className = "show";
  toast.textContent = msg;

  setTimeout(function () {
    toast.className = toast.className.replace("show", "");
  }, 1499);
}

function renderDeals(deals) {
  const dealsContainer = document.getElementById("deals-container");

  dealsContainer.innerHTML = "";
  deals.forEach((deal) => {
    const dealCard = document.createElement("div");
    dealCard.classList.add("deal-card");

    const dealName = document.createElement("div");
    // dealName.textContent = `${deal.first_name} ${deal.last_name}`;
    dealName.textContent = `${deal?.properties?.dealname}`;

    const container = document.createElement("div");
    container.classList.add("deal-card-container");
    const rightContainer = document.createElement("div");
    rightContainer.classList.add("deal-right-container");

    container.appendChild(dealName);
    container.appendChild(rightContainer);
    dealCard.appendChild(container);

    const fillButton = document.createElement("button");
    fillButton.classList.add("fill-button");
    fillButton.textContent = "Fill";
    fillButton.onclick = () => {
      chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
        if (tabs[0].url.includes("dealertrack")) {
          chrome.tabs.sendMessage(
            tabs[0].id,
            { type: "fillForm", deal },
            (response) => {
              if (chrome.runtime.lastError) {
                console.error(chrome.runtime.lastError);
                renderToast("Form filled successfully", "red");
              } else {
                console.log(response);
                renderToast("Form filled successfully", "green");
              }
            }
          );
        } else {
          renderToast("This is not dealertrack", "red");
        }
      });
    };

    rightContainer.appendChild(fillButton);

    dealsContainer.appendChild(dealCard);
  });
}

function isRequiredFormat(str) {
  try {
    JSON.parse(str);
  } catch (e) {
    return false;
  }

  const temp = JSON.parse(str);
  if (Array.isArray(temp)) return true;
  return false;
}

async function main() {
  let uid = localStorage.getItem("uid");
  const path = "https://app-f42d5nmmhq-uc.a.run.app";

  if (!uid) {
    uid = Date.now() + "," + Math.floor(Math.random() * 10000);
    localStorage.setItem("uid", uid);
  }

  const searchInput = document.getElementById("search-input");
  const searchBtn = document.getElementById("refresh-data");
  let deals;

  document.addEventListener("DOMContentLoaded", async function () {
    try {
      if (
        localStorage.getItem("deals") &&
        isRequiredFormat(localStorage.getItem("deals"))
      )
        deals = JSON.parse(localStorage.getItem("deals"));
    } catch (e) {
      console.log("Error ", e);
    }

    renderDeals(deals);
  });

  searchBtn.addEventListener("click", async (e) => {
    document.getElementById("deals-container").innerHTML =
      '<div class="center"><div class="wave"></div><div class="wave"></div><div class="wave"></div><div class="wave"></div><div class="wave"></div><div class="wave"></div><div class="wave"></div><div class="wave"></div><div class="wave"></div><div class="wave"></div></div>';
    const dealname = searchInput.value;
    const response = await fetch(`${path}/search/deal`, {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
      },
      body: JSON.stringify({ uid, dealname }),
    });
    const result = await response.json();
    if (!result["isAuthorized"]) {
      chrome.tabs.create({ url: `${path}/install/${uid}` });
    }

    deals = result.deals;
    localStorage.setItem("deals", JSON.stringify(deals));
    renderDeals(deals);
  });
}

main();
