function formatNumber(phoneNumber) {
  if (!phoneNumber) return "";

  // Remove dashes, spaces, and brackets using regex
  let modifiedNum = phoneNumber.replace(/[-\s()]/g, "");

  // // Remove a preceding '1' if it exists
  modifiedNum = modifiedNum.replace(/^1/, "");

  // // Truncate or pad to 10 digits
  // const digitsOnly = formattedNumber.slice(0, 10).padEnd(10, '0');

  return "(" + modifiedNum;
}

function formatPostalCode(postal_code) {
  if (!postal_code) return "";
  return postal_code.replace(/\s+/g, "").toUpperCase();
}

(() => {
  const onChange = new Event("change", { bubbles: true });
  const onClick = new Event("click", { bubbles: true });

  function setStreetAddress(
    streetNumberId,
    streetNameId,
    streetTypeId,
    street_address
  ) {
    if (!street_address) {
      setInput(streetNumberId, "");
      setInput(streetNameId, "");
      setInput(streetTypeId, "");
      return;
    }
    const streetTypeMappings = {
      ST: ["ST", "STREET"],
      RD: ["RD", "ROAD"],
      DR: ["DR", "DRIVE"],
      AVE: ["AVE", "AVENUE"],
      LN: ["LN", "LANE"],
      CT: ["CT", "COURT"],
      CIR: ["CIR", "CIRCLE"],
      BLVD: ["BLVD", "BOULEVARD"],
      PL: ["PL", "PLACE"],
      TER: ["TER", "TERRACE"],
      PKWY: ["PKWY", "PARKWAY"],
      HTS: ["HTS", "HEIGHTS"],
    };

    let streetComponents = street_address.split(" ");
    let streetNumber = "";
    let streetName = "";
    let streetType = "";

    streetComponents.forEach((component, index) => {
      let upperComponent = component.toUpperCase();
      if (index === 0 && !isNaN(parseInt(component))) {
        streetNumber = component;
      } else if (
        Object.keys(streetTypeMappings).some((key) =>
          streetTypeMappings[key].includes(upperComponent)
        )
      ) {
        streetType = Object.keys(streetTypeMappings).find((key) =>
          streetTypeMappings[key].includes(upperComponent)
        );
      } else {
        streetName += component + " ";
      }
    });

    streetName = streetName.trim();
    streetNumber = streetNumber.trim();

    setInput(streetNumberId, streetNumber);
    setInput(streetNameId, streetName);
    setInput(streetTypeId, streetType);
  }

  function getProvinceValue(provinceStr) {
    if (!provinceStr) return "";
    const provinces = {
      AB: "1",
      ALBERTA: "1",
      BC: "2",
      "BRITISH COLUMBIA": "2",
      MB: "3",
      MANITOBA: "3",
      NB: "4",
      "NEW BRUNSWICK": "4",
      NL: "5",
      NEWFOUNDLAND: "5",
      NT: "7",
      "NORTHWEST TERRITORIES": "7",
      NS: "6",
      "NOVA SCOTIA": "6",
      NU: "8",
      NUNAVUT: "8",
      ON: "9",
      ONTARIO: "9",
      PE: "10",
      "PRINCE EDWARD ISLAND": "10",
      QC: "11",
      QUEBEC: "11",
      SK: "12",
      SASKATCHEWAN: "12",
      YT: "13",
      YUKON: "13",
    };

    // Sanitize the input string
    const sanitizedStr = provinceStr.trim().toUpperCase();

    // Return the corresponding value or an empty string if not found
    return provinces[sanitizedStr] || "";
  }

  function getAddressType(address) {
    if (!address) return "ST";
    let addressType = "ST"; // default to "Street"
    if (/\b(P\.O\.?|Post\sOffice)\b/i.test(address)) {
      addressType = "PB"; // set to "Postal Box"
    }
    return addressType;
  }

  function getEmploymentType(isSelfEmployed = false) {
    return isSelfEmployed ? "Self-Employed" : "Professional";
  }

  function setDuration(duration, yearId, monthId) {
    if (!duration) {
      setInput(yearId, "");
      setInput(monthId, "");
      return;
    }
    let year, month;
    if (duration.toLowerCase().match("over")) {
      year = 2;
      month = 5;
    } else {
      year = 1;
      month = 0;
    }

    setInput(yearId, year);
    setInput(monthId, month);
  }

  function setInput(id, value) {
    const element = document.getElementById(id);
    element.value = value != null ? value : "";
    element.dispatchEvent(onChange);
  }

  function setDoB(birth_date) {
    if (!birth_date) {
      return;
    }
    setInput("ctl20_ctl16_ctl00_txtDateofBirth_MM", birth_date.split("-")[1]);
    setInput("ctl20_ctl16_ctl00_txtDateofBirth_DD", birth_date.split("-")[2]);
    setInput("ctl20_ctl16_ctl00_txtDateofBirth_YYYY", birth_date.split("-")[0]);
  }

  function setDetails(deal) {
    const props = [
      {
        id: "ctl20_ctl16_ctl00_txtFirstName",
        value: deal["first_name"],
      },
      { id: "ctl20_ctl16_ctl00_txtMiddleName", value: deal["middle_name"] },
      { id: "ctl20_ctl16_ctl00_txtLastName", value: deal["last_name"] },
      { id: "ctl20_ctl16_ctl00_txtSIN", value: deal["sin"] },
      { id: "ctl20_ctl16_ctl00_txtEmail", value: deal["email_address"] },
      { id: "ctl20_ctl17_ctl00_txtSuiteNumber", value: deal["unit__"] },
      { id: "ctl20_ctl17_ctl00_txtCity", value: deal["city"] },
      {
        id: "ctl20_ctl17_ctl00_ddlProvince",
        value: getProvinceValue(deal["province"]),
      },
      {
        id: "ctl20_ctl16_ctl00_txtPhone",
        value: formatNumber(deal["phone_number"]),
      },
      {
        id: "ctl20_ctl16_ctl00_txtMobilePhone",
        value: formatNumber(deal["phone_number"]),
      },
      { id: "ctl20_ctl16_ctl00_ddlGender", value: "MALE" },
      { id: "ctl20_ctl16_ctl00_ddlMaritalStatus", value: "SG" },
      { id: "ctl20_ctl16_ctl00_ddlLanguage", value: "en-CA" },
      {
        id: "ctl20_ctl17_ctl00_ddlAddressType",
        value: getAddressType(deal["street_address"]),
      },
      { id: "ctl20_ctl19_ctl00_ddlHome", value: "RE" },
      { id: "ctl20_ctl19_ctl00_txtMortgageHolder", value: "Private" },
      {
        id: "ctl20_ctl19_ctl00_txtMonthlyPayment",
        value: 300,
      },
      {
        id: "ctl20_ctl22_ctl00_txtGrossIncome",
        value: deal["monthly_gross_income"],
      },
      { id: "ctl20_ctl22_ctl00_ddlIncomeBasis", value: "12" },
      { id: "ctl20_ctl20_ctl00_ddlStatusCurEmp", value: "FT" },
      {
        id: "ctl20_ctl20_ctl00_ddlTypeCurEmp",
        value: getEmploymentType(deal["is_self_employed"]),
      },
      {
        id: "ctl20_ctl20_ctl00_txtOccupationCurEmp",
        value: deal["job_title"],
      },
      {
        id: "ctl20_ctl20_ctl00_txtEmployerCurEmp",
        value: deal["employer_name"],
      },
      {
        id: "ctl20_ctl20_ctl00_txtTelephoneCurEmp",
        value: formatNumber(deal["employer_phone_number"]),
      },
      {
        id: "ctl20_ctl20_ctl00_txtSuiteNumberCurEmp",
        value: deal["employer_unit__"],
      },
      { id: "ctl20_ctl20_ctl00__txtCityCurEmp", value: deal["employer_city"] },
      {
        id: "ctl20_ctl20_ctl00__ddlProvinceCurEmp",
        value: getProvinceValue(deal["employer_province"]),
      },
      {
        id: "ctl20_ctl20_ctl00_txtPostalCodeCurEmp",
        value: formatPostalCode(deal["employer_postal_code"]),
      },
      {
        id: "ctl20_ctl20_ctl00_ddlAddressTypeCurEmp",
        value: getAddressType(deal["employer_street_address"]),
      },
      {
        id: "ctl20_ctl21_ctl00_ddlType",
        value: getEmploymentType(),
      },
      {
        id: "ctl20_ctl21_ctl00_txtTelephone",
        value: formatNumber(deal["previous_employer_phone_number"]),
      },
      {
        id: "ctl20_ctl21_ctl00_txtPostalCode",
        value: formatPostalCode(deal["previous_employer_postal_code"]),
      },
      // {
      //   id: "ctl20_ctl20_ctl00_ddlTypeCurEmp",
      //   value: getEmploymentType(deal["is_self_employed"]),
      // },
      // {
      //   id: "ctl20_ctl20_ctl00_txtOccupationCurEmp",
      //   value: deal["job_title"],
      // },
      {
        id: "ctl20_ctl21_ctl00_txtSuiteNumber",
        value: deal["previous_employer_unit__"],
      },
      {
        id: "ctl20_ctl21_ctl00_txtCity",
        value: deal["previous_employer_city"],
      },
      {
        id: "ctl20_ctl21_ctl00_ddlProvince",
        value: getProvinceValue(deal["previous_employer_province"]),
      },
      {
        id: "ctl20_ctl21_ctl00_ddlAddressType",
        value: getAddressType(deal["previous_employer_street_address"]),
      },
      {
        id: "ctl20_ctl17_ctl00_txtPostalCode",
        value: formatPostalCode(deal["postal_code"]),
      },
      {
        id: "ctl20_ctl18_ctl00_txtPostalCode",
        value: formatPostalCode(deal["previous_postal_code"]),
      },
      {
        id: "ctl20_ctl18_ctl00_ddlAddressType",
        value: getAddressType(deal["previous_street_address"]),
      },
      {
        id: "ctl20_ctl18_ctl00_txtSuiteNumber",
        value: deal["previous_unit__"],
      },
      {
        id: "ctl20_ctl18_ctl00_txtCity",
        value: deal["previous_city"],
      },
      {
        id: "ctl20_ctl18_ctl00_ddlProvince",
        value: getProvinceValue(deal["previous_province"]),
      },
      {
        id: "ctl20_ctl21_ctl00_txtEmployer",
        value: deal["previous_employer_name"],
      },
      { id: "ctl20_ctl21_ctl00_ddlStatus", value: "FT" },
      {
        id: "ctl20_ctl21_ctl00_txtOccupation",
        value: deal["previous_employer_position"],
      },
    ];

    console.log("Setting DoB");

    setDoB(deal["birth_date"]);

    console.log("Setting Duration");

    setDuration(
      deal["duration_at_address"],
      "ctl20_ctl17_ctl00_CDurationCurrentAddress_Y",
      "ctl20_ctl17_ctl00_CDurationCurrentAddress_M"
    );
    // setDuration(
    //   deal["duration_at_address"],
    //   "ctl20_ctl18_ctl00_CDurationPreviousAddress_Y",
    //   "ctl20_ctl18_ctl00_CDurationPreviousAddress_M"
    // );
    setDuration(
      deal["employment_lengthnew"],
      "ctl20_ctl20_ctl00_CDurationCurrentEmployerAddress_Y",
      "ctl20_ctl20_ctl00_CDurationCurrentEmployerAddress_M"
    );
    setDuration(
      deal["previous_employment_length"],
      "ctl20_ctl21_ctl00_CDurationPreviousEmployerAddress_Y",
      "ctl20_ctl21_ctl00_CDurationPreviousEmployerAddress_M"
    );

    console.log("Setting Street Address");

    setStreetAddress(
      "ctl20_ctl17_ctl00_txtStreetNumber",
      "ctl20_ctl17_ctl00_txtStreetName",
      "ctl20_ctl17_ctl00_ddlStreetType",
      deal["street_address"]
    );
    setStreetAddress(
      "ctl20_ctl18_ctl00_txtStreetNumber",
      "ctl20_ctl18_ctl00_txtStreetName",
      "ctl20_ctl18_ctl00_ddlStreetType",
      deal["previous_street_address"]
    );
    setStreetAddress(
      "ctl20_ctl20_ctl00_txtStreetNumberCurEmp",
      "ctl20_ctl20_ctl00_txtStreetNameCurEmp",
      "ctl20_ctl20_ctl00_ddlStreetTypeCurEmp",
      deal["employer_street_address"]
    );
    setStreetAddress(
      "ctl20_ctl21_ctl00_txtStreetNumber",
      "ctl20_ctl21_ctl00_txtStreetName",
      "ctl20_ctl21_ctl00_ddlStreetType",
      deal["previous_employer_street_address"]
    );

    //set all directly filled input elements
    props.forEach((prop) => {
      setInput(prop.id, prop.value);
    });
  }

  function fillForm(deal) {
    console.log("Filling form with deal data: ", deal);

    setDetails(deal);
  }

  chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
    if (request.type === "fillForm") {
      fillForm(request.deal.properties);
      sendResponse({ success: true });
    }
  });

  console.log("Content script loaded and running.");
})();
